#include <QCoreApplication>
#include <QTextStream>
#include <QDateTime>

#include <QPrinterInfo>
#include <QPainter>

class CTimer
{
public:
    CTimer( const QString& sTitle );
    ~CTimer();

private:
    QDateTime _qStart;
    QString _sTitle;
};

QTextStream qOut( stdout );

#define OdaPI 3.14159265358979323846
#define OdaToDegree(rad) ((rad)/OdaPI*180.0)

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    QString sPrinterName = "HP Universal Printing PCL 5"; //"HP Designjet Z5200ps PS3";

    QPrinterInfo info;
#if ( QT_VERSION >= QT_VERSION_CHECK(5, 3, 0) )
    info = QPrinterInfo::printerInfo( sPrinterName );
#else
    QList<QPrinterInfo> printers = QPrinterInfo::availablePrinters();
    for ( int i = 0; i < printers.size(); ++i )
    {
        if ( printers[ i ].printerName() == sPrinterName )
            info = printers[ i ];
    }
#endif
    QPrinter *pPrinter = new QPrinter( info, QPrinter::HighResolution );
    if ( pPrinter )
    {
        pPrinter->setDocName( "ellipses" );
        pPrinter->setOutputFileName( "ellipses_test.ps" );
        pPrinter->setOutputFormat( QPrinter::NativeFormat );
        pPrinter->setFullPage( true );
        pPrinter->setNumCopies( 1 );
        pPrinter->setResolution( 600 );
#if ( QT_VERSION >= QT_VERSION_CHECK(5, 3, 0) )
        pPrinter->setPageSize( QPageSize( QPageSize::A0 ) );
#else
        pPrinter->setPaperSize( QPrinter::A0 );
#endif

        CTimer timer( "Printing ellipses");

        QPainter p( pPrinter );
        p.setClipRect( QRectF( 2000, 1000, 26000, 18000 ) );
        p.save();

        QPen pen( Qt::black );
        pen.setStyle( Qt::SolidLine );
        pen.setCapStyle( Qt::FlatCap );
        pen.setJoinStyle( Qt::MiterJoin );
        pen.setWidth( 0 );
        pen.setCosmetic( true );
        p.setPen( pen );

        QBrush brush( Qt::black, Qt::SolidPattern );
        p.setBrush( brush );

        p.setClipping( true );
        QPainterPath path;
        {
            QPointF points[ 5 ] = { QPointF(2000, 1000), QPointF(2000, 19000), QPointF(28000, 19000), QPointF(28000, 1000), QPointF(2000, 1000) };
            QPolygonF polygon;
            polygon.resize( 5 );
            for ( int i = 0; i < 5; ++i )
                polygon[ i ] = points[ i ];
            path.addPolygon( polygon );
        }
        p.setClipPath( path, Qt::IntersectClip );

        const QPointF center( 12000, 8000 );
        const QPointF major(  0.82580098187352990, -0.56396164615753253 );
        const QPointF minor( -0.56396164615753264, -0.82580098187352979 );
        const double majorR = 3451.3567375455118;
        const double minorR = 1618.1020172054223;
        const double start = 0.00000000000000000;
        const double span = 6.2831853071795862;
        const int count = 10;
        for ( int i = 0; i < count; ++i )
        {
            p.save();

            p.setBrush( Qt::NoBrush );

            {
                QTransform mT;
                mT.translate( center.x() + i, center.y() );
                p.setTransform( mT, true );
            }

            {
                QTransform mT;
                mT.setMatrix( major.x(), major.y(), 0, -minor.x(), -minor.y(), 0, 0, 0, 1 );
                //if ( !mT.isInvertible() )
                //{
                //    p.restore();
                //    break;;
                //}
                p.setTransform( mT, true );
            }

            QRectF rect(
                -majorR,
                -minorR,
                2.0 * majorR,
                2.0 * minorR );

            QPainterPath path;
            path.arcMoveTo( rect, OdaToDegree( start ) );
            path.arcTo( rect, OdaToDegree( start ), OdaToDegree( span ) );
            p.drawPath( path );

            p.restore();
        }

        p.restore();
    }

    return a.exec();
}

//---------------------------------------------------------------------------------------------------------------------

CTimer::CTimer( const QString& sTitle ) : _sTitle( sTitle)
{
    _qStart = QDateTime::currentDateTime();
}

CTimer::~CTimer()
{
    qOut << _sTitle << ": " << _qStart.msecsTo( QDateTime::currentDateTime() ) << "ms" << endl;
}
